/*
	Programme de chenillard illustrant l'utilisation d'un timer comme base de temps.
	Les LEDs sont connectes entre les pins du port D et la masse (=> allumes lorque
	l'I/O est  1)
	
	Le principe est de faire "tourner" le timer 0, et de rcuprer l'interruption
	 chaque passage de 255  0.
	
	Etant donn qu'on veut obtenir une cadence de dfilement visible (envrion 2/10e
	de seconde, il faut cumuler plusieurs divisions du temps.
	1/ 	on va appliquer un prescale de 1024  l'horloge processeur. Vu qu'elle
		est  8 MHz, la priode de base est de 125ns. Le precale va donc tablir la
		priode de l'incrment du compteur  125nS * 1024, soit 128s. Un tour complet
		de compteur va donc durer 128s * 256, soit 33ms environ. C'est la priode avec
		laquelle l'interruption sera dclenche
	2/ 	on va changer l'tat des sorties toutes les n interruptions, de manire  avoir
		une cadence de dfilement visible

	Cible : ATMega8
	Quartz : 8MHz
	Compilateur : avr-gcc (WInAVR)
*/

#include <avr/io.h> 
#include <avr/interrupt.h>

// compteur du nombre d'interruptions timer
volatile int tmr_cnt = 0 ;

// vu le prescaling et l'tendue du compteur, un intervalle de 100ms correspond  3 interruptions
#define _100_MS		 (3 * 64)
#define _1_S		 (30 * 4)

unsigned char digits[] = {
	0b00111111,
	0b00000110,
	0b01011011,
	0b01001111,
	0b01100110,
	0b01101101,
	0b01111101,
	0b00000111,
	0b01111111,
	0b01101111, 
	0b01110111,
	0b01111100,
	0b00111001,
	0b01011110,
	0b01111001,
	0b01110001
} ;

unsigned char digit_num = 0 ;
unsigned char digit_sel ;
unsigned char digit_value[4] = {0,0,0,0} ;
unsigned int value = 0 ;
 
/*
	Interrupt handler pour l'overflow du timer 0 (T=33ms)
*/
SIGNAL (SIG_OVERFLOW0) {

	// si on est arriv au nombre ncessaire  l'obtention du dlai souhait,
	// il est temps de changer de motif
	if (tmr_cnt == 0) {
		value++ ;
		digit_value[0] = value % 10 ;
		digit_value[1] = (value % 100) / 10 ;
		digit_value[2] = (value % 1000) / 100 ;
		digit_value[3] = (value % 10000) / 1000 ;
	}

	PORTA = digits[digit_value[digit_num]] ;
	PORTB = digit_sel ;
	
	digit_num = (digit_num + 1) & 0x03 ;
	digit_sel = 1 << digit_num ;

	// mise  jour du compteur d'interruptions
	tmr_cnt =  (tmr_cnt + 1) % _1_S ;
}

/*
	Initialisation des ports
*/ 
void port_init(void) {
 	DDRA  = 0xFF ;
 	DDRB  = 0xFF ;
}

/*
	Initialisation des priphriques
*/
void init_devices(void) {
 	cli(); 						// inhibition des interruptions (pour avoir la paix)
 	port_init();				// initialisation de prts

	// MCU Control Register
	// - interruptions externes sur niveau bas de la ligne
	// - sleep mode dsactiv
 	MCUCR = 0x00;			
 	
 	// General Interrupt Control
 	// - INT0 et INT1 inactives
 	GICR = 0x00;
 	
 	// Timer/Counter Interrupt Mask
 	// - activation de l'interrupt d'overflow du timer 0
 	TIMSK = 0x01; 	
 	
 	// Timer/Counter 0 Control Register
 	// - pre-scaling rgl sur 1024
 	//TCCR0 = 0x05;
 	TCCR0 = 0x03 ;	  
 	
 	sei();  	   	  			// autorisation des interruptions
}

//- --------------------------------------------------------------------------------
int main(void) {
	init_devices();
 
	 // rien  ajouter : tout est fait dans l'interrupt handler
	 return 0 ;
}

