/*
	Programme de chenillard illustrant l'utilisation d'un timer comme base de temps.
	Les LEDs sont connectes entre les pins du port D et la masse (=> allumes lorque
	l'I/O est  1)
	
	Le principe est de faire "tourner" le timer 0, et de rcuprer l'interruption
	 chaque passage de 255  0.
	
	Etant donn qu'on veut obtenir une cadence de dfilement visible (envrion 2/10e
	de seconde, il faut cumuler plusieurs divisions du temps.
	1/ 	on va appliquer un prescale de 1024  l'horloge processeur. Vu qu'elle
		est  8 MHz, la priode de base est de 125ns. Le precale va donc tablir la
		priode de l'incrment du compteur  125nS * 1024, soit 128s. Un tour complet
		de compteur va donc durer 128s * 256, soit 33ms environ. C'est la priode avec
		laquelle l'interruption sera dclenche
	2/ 	on va changer l'tat des sorties toutes les n interruptions, de manire  avoir
		une cadence de dfilement visible

	Cible : ATMega8
	Quartz : 8MHz
	Compilateur : avr-gcc (WInAVR)
*/

#include <avr/io.h> 
#include <avr/interrupt.h>

// compteur du nombre d'interruptions timer
volatile int tmr_cnt = 0 ;

// vu le prescaling et l'tendue du compteur, un intervalle de 100ms correspond  3 interruptions
#define _100_MS		 3

// la liste des patterns d'allumage des LEDs
unsigned char patterns[] = {
	0b10000000,
	0b01000000,
	0b00100000,
	0b00010000,
	0b00001000,
	0b00000100,
	0b00000010,
	0b00000001,
	0b00000010,
	0b00000100,
	0b00001000,
	0b00010000,
	0b00100000,
	0b01000000,
	0b10000000,
	0b00100000,
	0b00001000,
	0b00000010,
	0b00000001,
	0b00000100,
	0b00010000,
	0b01000000,
	0b00000000,
	0b10000000,
	0b11000000,
	0b11100000,
	0b11110000,
	0b11111000,
	0b11111100,
	0b11111110,
	0b11111111,
	0b01111111,
	0b00111111,
	0b00011111,
	0b00001111,
	0b00000111,
	0b00000011,
	0b00000001,
	0b00000000,
	0b10101010,
	0b10101010,
	0b01010101,
	0b01010101,
	0b10101010,
	0b10101010,
	0b01010101,
	0b01010101,
	0b10101010,
	0b10101010,
	0b01010101,
	0b01010101,
	0b11111111,
	0b00000000,
	0b11111111,
	0b00000000,
	0b11111111,
	0b00000000
} ;

// index du pattern courant
unsigned char pattern = 0 ;
 
/*
	Interrupt handler pour l'overflow du timer 0
*/
ISR (SIG_OVERFLOW0) {
	// mise  jour du compteur d'interruptions
	tmr_cnt++ ;
	// si on est arriv au nombre ncessaire  l'obtention du dlai souhait,
	// il est temps de changer de motif
	if (tmr_cnt == _100_MS) {
		// remise  0 du compteur d'interruptions
		tmr_cnt = 0 ;
		
		// on modifie l'tat des sorties avec le pattern courant
		PORTD = patterns[pattern] ;
		
		if (pattern > 0) {
			PORTC = patterns[pattern - 1] ;
		} else {
			PORTC = patterns[sizeof(patterns) - 1] ;
		} 
		if (pattern > 1) {
			PORTB = patterns[pattern - 2] ;
		} else {
			PORTB = patterns[pattern + sizeof(patterns) - 2] ;
		} 
		if (pattern > 2) {
			PORTA = patterns[pattern - 3] ;
		} else {
			PORTA = patterns[pattern + sizeof(patterns) - 3] ;
		} 
		
		// avance de l'index du pattern, avec rebouclage si en fin de squence
		pattern = (pattern + 1) % sizeof(patterns) ;
	}
}

/*
	Initialisation des ports
*/ 
void port_init(void) {
	// configuration du port D
 	// - toutes les I/O en sortie
 	DDRA  = 0xFF ;
 	DDRB  = 0xFF ;
 	DDRC  = 0xFF ;
 	DDRD  = 0xFF ;
	// - toutes les LEDs teintes
 	PORTD = 0x00 ;
}

/*
	Initialisation des priphriques
*/
void init_devices(void) {
 	cli(); 						// inhibition des interruptions (pour avoir la paix)
 	port_init();				// initialisation de prts

	// MCU Control Register
	// - interruptions externes sur niveau bas de la ligne
	// - sleep mode dsactiv
 	MCUCR = 0x00;			
 	
 	// General Interrupt Control
 	// - INT0 et INT1 inactives
 	GICR = 0x00;
 	
 	// Timer/Counter Interrupt Mask
 	// - activation de l'interrupt d'overflow du timer 0
 	TIMSK = 0x01; 	
 	
 	// Timer/Counter 0 Control Register
 	// - pre-scaling rgl sur 1024
 	TCCR0 = 0x05;	  
 	
 	sei();  	   	  			// autorisation des interruptions
}

//- --------------------------------------------------------------------------------
int main(void) {
	init_devices();
 
	 // rien  ajouter : tout est fait dans l'interrupt handler
	 return 0 ;
}

